﻿using DataStruct;
using MyDiskInfo.DataStruct;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Utils;
using WCFServer;

namespace MyDiskInfo
{
    /// <summary>
    /// 固态硬盘信息显示控件
    /// </summary>
    public partial class DiskInfoCtrl : UserControl
    {
        #region 字段
        private System.Timers.Timer _timer = null;
        private IClientServer _client = new ClientServer();
        private double _width = 140;
        private double _height = 30;
        private int _interval = 1000; //刷新间隔
        private DiskModel _diskInfo = null;
        private CpuModel _cpuInfo = null;
        private DiskInfoViewModel _model = new DiskInfoViewModel();
        #endregion

        #region 构造函数
        public DiskInfoCtrl()
        {
            InitializeComponent();

            try
            {
                LogUtil.Log("客户端开始启动");

                //尺寸
                this.Width = _width;
                this.Height = _height;

                this.DataContext = _model;

                ShowCpuInfo();

                _timer = new System.Timers.Timer();
                _timer.Interval = _interval;
                _timer.Elapsed += (s, e) =>
                {
                    ShowCpuInfo();
                };
                _timer.Start();

                LogUtil.Log("客户端启动成功");
            }
            catch (Exception ex)
            {
                LogUtil.Error(ex, "构造函数DiskInfoCtrl出错");
            }
        }
        #endregion

        #region 显示CPU信息
        private void ShowCpuInfo()
        {
            BackWork.Run(() =>
            {
                _cpuInfo = _client.GetCpuInfo();
            }, () =>
            {
                _model.Info = _cpuInfo.Load.ToString("0.00").PadLeft(6) + "% " + _cpuInfo.Clock.ToString("0.00").PadLeft(4) + " " + _cpuInfo.Temperature.ToString("0").PadLeft(3);
            });
        }
        #endregion

        #region 显示硬盘信息
        /// <summary>
        /// 显示硬盘信息
        /// </summary>
        private void ShowDiskInfo()
        {
            BackWork.Run(() =>
            {
                _diskInfo = _client.GetDiskInfo();
            }, () =>
            {
                _model.Info = string.Format("{0}G {1}G {2}PE", _diskInfo.LBA_TodayWrite, _diskInfo.LBA_Write, _diskInfo.PE);
            });
        }
        #endregion

        #region 释放资源
        public void ClearResources()
        {
            if (_timer != null)
            {
                _timer.Stop();
                _timer.Close();
                _timer = null;
                LogUtil.Log("客户端释放资源");
            }
        }
        #endregion

    }

    #region DiskInfoViewModel
    public class DiskInfoViewModel : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        private string _Info;
        public string Info
        {
            get { return _Info; }
            set
            {
                _Info = value;
                OnPropertyChanged("Info");
            }
        }

        public void OnPropertyChanged(string name)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(name));
            }
        }
    }
    #endregion

}
