﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using Utils;

/**
 * 使用方法：

BackWork.Run(() => //DoWork
{

}, () => //RunWorkerCompleted
{

}, (ex) => //错误处理
{

});
 
*/

namespace MyDiskInfo
{
    /// <summary>
    /// BackgroundWorker封装
    /// 用于简化代码
    /// </summary>
    public class BackWork
    {
        /// <summary>
        /// 执行
        /// </summary>
        /// <param name="doWork">DoWork</param>
        /// <param name="workCompleted">RunWorkerCompleted</param>
        /// <param name="errorAction">错误处理</param>
        public static void Run(Action doWork, Action workCompleted, Action<Exception> errorAction = null)
        {
            bool isDoWorkError = false;
            Exception doWorkException = null;
            BackgroundWorker worker = new BackgroundWorker();
            worker.DoWork += (s, e) =>
            {
                try
                {
                    doWork();
                }
                catch (Exception ex)
                {
                    isDoWorkError = true;
                    doWorkException = ex;
                    LogUtil.Error(ex, "BackWork.Run error");
                }
            };
            worker.RunWorkerCompleted += (s, e) =>
            {
                if (!isDoWorkError)
                {
                    try
                    {
                        if (workCompleted != null) workCompleted();
                    }
                    catch (Exception ex)
                    {
                        errorAction(ex);
                        LogUtil.Error(ex, "BackWork.Run error");
                    }
                }
                else
                {
                    errorAction(doWorkException);
                }
            };
            worker.RunWorkerAsync();
        }

    }
}