﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace MyDiskInfo.Service
{
    /// <summary>
    /// 线程工具类
    /// </summary>
    public class ThreadUtil
    {
        /// <summary>
        /// 使用的逻辑处理器数
        /// </summary>
        private static int _ProcessorCount;
        /// <summary>
        /// 线程数
        /// </summary>
        private static int _ThreadCount = 0;
        private static object _Lock = new object();
        private static List<Task> _TaskList = new List<Task>();

        static ThreadUtil()
        {
            _ProcessorCount = Environment.ProcessorCount * 2 / 4; //使用的逻辑处理器数
            if (_ProcessorCount < 1) _ProcessorCount = 1;
        }

        public static void Run(Action<object> doWork, object arg, Action<Exception> errorAction)
        {
            Task task = Task.Factory.StartNew((obj) =>
            {
                while (_ThreadCount >= _ProcessorCount)
                {
                    Thread.Sleep(1);
                }
                lock (_Lock)
                {
                    _ThreadCount++;
                }

                try
                {
                    doWork(obj);
                }
                catch (Exception ex)
                {
                    errorAction(ex);
                }

                lock (_Lock)
                {
                    _ThreadCount--;
                }
            }, arg);
            _TaskList.Add(task);
        }

        public static void WaitAll()
        {
            Task.WaitAll(_TaskList.ToArray());
        }
    }
}
